/**
 * Module dependencies
 */

const resolve = require('../utils/resolve')
const readdir = require('../utils/readdir')
const Changed = require('../utils/changed')
const decrypt = require('../utils/decrypt')
const encrypt = require('../utils/encrypt')
const flatten = require('lodash.flatten')
const Prompt = require('../utils/prompt')
const dice = require('dice-coefficient')
const extend = require('deep-extend')
const glob = require('../utils/glob')
const assert = require('assert')
const fuzzy = require('fuzzy')
const chalk = require('chalk')
const { EOL } = require('os')
const path = require('path')
const fs = require('co-fs')
const co = require('co')


/**
 * Commands
 */

const Commands = require('../commands')

/**
 * Export `run`
 */

module.exports = co.wrap(run)

/**
 * Initialize `run`
 */

function * run ({ root, paths = [], commands = [], plan = false }) {
  assert.ok(commands.length, 'You must specify atleast 1 command')
  commands = Array.isArray(commands) ? commands : [ commands ]
  paths = Array.isArray(paths) ? paths : [ paths ]
  paths = paths.map(p => path.resolve(root, p))

  // resolve from thimble.yml files
  let cmds = yield resolve(paths)

  // fuzzy match on the command
  let matches = commands.map(command => {
    let matches = fuzzy.filter(command, cmds, {
      extract: (m) => m.dirname + '/' + m.command
    })

    // no matches
    if (!matches.length) {
      throw new Error(`Couldn't find the command "${command}". Did you install this scaffolding?`)
    } else if (matches.length > 1) {
      let str = matches.map(m => m.original.path).join('", "')
      throw new Error(`Not sure which command to run: "${str}". Can you be a little more specific?`)
    }

    return matches[0].original
  })

  // optionally load the state
  const statepath = path.join(root, 'thimble.state')
  const has_state = yield fs.exists(statepath)
  let password = null
  let state = {}

  // do we already have state?
  if (has_state) {
    const contents = yield fs.readFile(statepath)
    password = yield Prompt.password(`What's the password to your thimble.state file?`)
    state = JSON.parse(decrypt(contents, password))
  }

  // get the time
  let time = Date.now()

  // run each command in order
  for (let { command, path: fullpath } of matches) {
    let directory = path.dirname(fullpath)
    let thimble = require(fullpath)

    // setup the thimble state namespace
    let namespace = path.relative(root, directory)
    let command_state = extend({}, state[namespace] || {})

    // TODO: get this out of here
    console.error()
    console.error(`  ${chalk.magenta('RUNNING')} ${command}`)

    // run through the commands
    let context = { root, directory, paths, commands, plan }
    yield thimble(Commands(context), command_state, context)
    if (Object.keys(command_state).length) state[namespace] = command_state
  }

  // filesystem after
  let changed = yield Changed(yield readdir(root), time)
  changed = changed.map(p => path.relative(root, p))

  // update the state and re-encrypt
  if (has_state) {
    let encrypted = encrypt(JSON.stringify(state, true, 2), password)
    yield fs.writeFile(statepath, encrypted)
  }

  return {
    changed,
    state
  }
}
