/**
 * Module Dependencies
 */

let invisibles = require('invisibles')
let diff = require('diff')
let path = require('path')

/**
 * Export `Diff`
 */

module.exports = Diff

/**
 * Default color map.
 */

let colors = {
  pass: 90,
  fail: 31,
  'bright pass': 92,
  'bright fail': 91,
  'bright yellow': 93,
  pending: 36,
  suite: 0,
  'error title': 0,
  'error message': 31,
  'error stack': 90,
  checkmark: 32,
  fast: 90,
  medium: 33,
  slow: 31,
  green: 32,
  light: 90,
  'diff gutter': 90,
  'diff added': 32,
  'diff removed': 31
}

/**
 * Diff
 */

function Diff (left_file, right_file, root) {
  return unified(
    left_file,
    right_file,
    root
  )
}

/**
 * Returns a unified diff between two strings.
 *
 * @api private
 * @param {Error} err with actual/expected
 * @param {boolean} escape
 * @return {string} The diff.
 */

function unified (left, right, root) {
  var indent = ''
  function cleanUp(line) {
    line = invisibles(line).replace(/[·¬‣]/g, function (m) {
      return colorLines('diff gutter', m)
    })

    if (line[0] === '+') {
      return indent + colorLines('diff added', line)
    }
    if (line[0] === '-') {
      return indent + colorLines('diff removed', line)
    }
    if (line.match(/\@\@/)) {
      return null
    }
    if (line.match(/\\ No newline/)) {
      return null
    }
    return indent + line
  }
  function notBlank(line) {
    return typeof line !== 'undefined' && line !== null
  }
  var msg = diff.createPatch('string', left.contents.toString(), right.contents.toString())
  var lines = msg.split('\n').splice(4)
  return '\n'
    + indent + colorLines('medium', 'ORIGINAL: ' + path.relative(root, left.path)) + '\n'
    + indent + colorLines('medium', 'NEW FILE: ' + path.relative(root, right.path))
    + '\n\n'
    + lines.map(cleanUp).filter(notBlank).join('\n')
}

/**
 * Color `str` with the given `type`,
 * allowing colors to be disabled,
 * as well as user-defined color
 * schemes.
 *
 * @param {string} type
 * @param {string} str
 * @return {string}
 * @api private
 */

function color (type, str) {
  return '\u001b[' + colors[type] + 'm' + str + '\u001b[0m'
}

/**
 * Color lines for `str`, using the color `name`.
 *
 * @api private
 * @param {string} name
 * @param {string} str
 * @return {string}
 */

function colorLines (name, str) {
  return str.split('\n').map(function(str) {
    return color(name, str)
  }).join('\n')
}
