/**
 * Module Dependencies
 */

const indent = require('indent-string')
const extend = require('deep-extend')
const Prompt = require('cli-prompt')
const md = require('./markdown')
const chalk = require('chalk')
const mask = require('./mask')

/**
 * Export `Prompt`
 */

module.exports = { prompt, password, confirm }

/**
 * Prompt
 *
 * @param {String} str
 * @return {String}
 */

function * prompt (str, options) {
  const { required, defaults } = opts(options)
  let value = null

  if (defaults) {
    const def = chalk.gray(`(${defaults})`)
    str = `${str} ${def} `
  }

  do { value = yield input(str) }
  while (required && !value)

  return value || defaults
}

/**
 * Password
 *
 * @param {String} str
 * @return {String}
 */

function * password (str, options) {
  const { required, defaults } = opts(options)
  let value = null

  if (defaults) {
    const def = chalk.gray(`(${mask(defaults)})`)
    str = `${str} ${def} `
  }

  do { value = yield input(str, true) }
  while (required && !value)

  return value || defaults
}

/**
 * Confirm the input
 *
 * @param {String} input
 * @return {String}
 */

function * confirm (str, options) {
  const { required, defaults } = opts(options)
  let value = null

  if (defaults) {
    str += ` ${chalk.gray(`(${defaults})`)} `
  } else {
    str += ` ${chalk.gray(`[y|n]`)} `
  }

  do { value = boolean(yield input(str)) }
  while (required && value === null)

  return value === null ? defaults : value
}

/**
 * Inputs
 *
 * @param {String} input
 * @return {String}
 */

function * input (input, hidden = false) {
  return yield (fn) => {
    console.error()
    Prompt(indent(chalk.yellow('[?]') + ' ' + md(input), 2) + ' ', hidden, (v) => fn(null, v), (e) => fn(e))
  }
}

/**
 * Boolean
 *
 * @param {String} value
 * @return {Boolean}
 */

function boolean (value) {
  if (/^(yes|ok|ack|true|y)$/i.test(value)) return true
  if (/^(no|nope|nack|false|n)$/i.test(value)) return false
  return null
}

/**
 * Parse the options
 *
 * @param {Object} options
 * @return {Object}
 */

function opts (options) {
  let out = {}

  // configurations
  if (!options) {
    out = {}
  } else if (typeof options === 'string') {
    out.defaults = options
  } else {
    out = extend({}, options)
  }

  // required
  let required = out.required === undefined
    && out.defaults === undefined

  // defaults
  out.required = required || !!out.required

  return out
}
